@@ -17,9 +17,14 @@
 import argparse
 import os
 import sys
+import uuid
 
 import fixtures
+from keystoneclient import exceptions as ks_exc
+from keystoneclient import fixture as ks_fixture
 import mock
+import requests
+from requests_mock.contrib import fixture as rm_fixture
 import six
 
 from glanceclient import exc
@@ -28,12 +33,8 @@ from glanceclient import shell as openstack_shell
 #NOTE (esheffield) Used for the schema caching tests
 from glanceclient.v2 import schemas as schemas
 import json
-from tests import keystone_client_fixtures
 from tests import utils
 
-import keystoneclient
-from keystoneclient.openstack.common.apiclient import exceptions as ks_exc
-
 
 DEFAULT_IMAGE_URL = 'http://127.0.0.1:5000/'
 DEFAULT_USERNAME = 'username'
@@ -43,8 +44,8 @@ DEFAULT_TENANT_NAME = 'tenant_name'
 DEFAULT_PROJECT_ID = '0123456789'
 DEFAULT_USER_DOMAIN_NAME = 'user_domain_name'
 DEFAULT_UNVERSIONED_AUTH_URL = 'http://127.0.0.1:5000/'
-DEFAULT_V2_AUTH_URL = 'http://127.0.0.1:5000/v2.0/'
-DEFAULT_V3_AUTH_URL = 'http://127.0.0.1:5000/v3/'
+DEFAULT_V2_AUTH_URL = '%sv2.0' % DEFAULT_UNVERSIONED_AUTH_URL
+DEFAULT_V3_AUTH_URL = '%sv3' % DEFAULT_UNVERSIONED_AUTH_URL
 DEFAULT_AUTH_TOKEN = ' 3bcc3d3a03f44e3d8377f9247b0ad155'
 TEST_SERVICE_URL = 'http://127.0.0.1:5000/'
 
@@ -61,12 +62,24 @@ FAKE_V3_ENV = {'OS_USERNAME': DEFAULT_USERNAME,
                'OS_AUTH_URL': DEFAULT_V3_AUTH_URL,
                'OS_IMAGE_URL': DEFAULT_IMAGE_URL}
 
+TOKEN_ID = uuid.uuid4().hex
+
+V2_TOKEN = ks_fixture.V2Token(token_id=TOKEN_ID)
+V2_TOKEN.set_scope()
+_s = V2_TOKEN.add_service('image', name='glance')
+_s.add_endpoint(DEFAULT_IMAGE_URL)
+
+V3_TOKEN = ks_fixture.V3Token()
+V3_TOKEN.set_project_scope()
+_s = V3_TOKEN.add_service('image', name='glance')
+_s.add_standard_endpoints(public=DEFAULT_IMAGE_URL)
+
 
 class ShellTest(utils.TestCase):
     # auth environment to use
     auth_env = FAKE_V2_ENV.copy()
     # expected auth plugin to invoke
-    auth_plugin = 'keystoneclient.auth.identity.v2.Password'
+    token_url = DEFAULT_V2_AUTH_URL + '/tokens'
 
     # Patch os.environ to avoid required auth info
     def make_env(self, exclude=None):
@@ -78,6 +91,25 @@ class ShellTest(utils.TestCase):
         global _old_env
         _old_env, os.environ = os.environ, self.auth_env
 
+        self.requests = self.useFixture(rm_fixture.Fixture())
+
+        json_list = ks_fixture.DiscoveryList(DEFAULT_UNVERSIONED_AUTH_URL)
+        self.requests.get(DEFAULT_IMAGE_URL, json=json_list, status_code=300)
+
+        json_v2 = {'version': ks_fixture.V2Discovery(DEFAULT_V2_AUTH_URL)}
+        self.requests.get(DEFAULT_V2_AUTH_URL, json=json_v2)
+
+        json_v3 = {'version': ks_fixture.V3Discovery(DEFAULT_V3_AUTH_URL)}
+        self.requests.get(DEFAULT_V3_AUTH_URL, json=json_v3)
+
+        self.v2_auth = self.requests.post(DEFAULT_V2_AUTH_URL + '/tokens',
+                                          json=V2_TOKEN)
+
+        headers = {'X-Subject-Token': TOKEN_ID}
+        self.v3_auth = self.requests.post(DEFAULT_V3_AUTH_URL + '/auth/tokens',
+                                          headers=headers,
+                                          json=V3_TOKEN)
+
         global shell, _shell, assert_called, assert_called_anytime
         _shell = openstack_shell.OpenStackImagesShell()
         shell = lambda cmd: _shell.main(cmd.split())
@@ -184,80 +216,59 @@ class ShellTest(utils.TestCase):
             self.assertEqual('https://image:1234', args[0])
             self.assertEqual('mytoken', kwargs['token'])
 
-    def _assert_auth_plugin_args(self, mock_auth_plugin):
+    def _assert_auth_plugin_args(self):
         # make sure our auth plugin is invoked with the correct args
-        mock_auth_plugin.assert_called_once_with(
-            keystone_client_fixtures.V2_URL,
-            self.auth_env['OS_USERNAME'],
-            self.auth_env['OS_PASSWORD'],
-            tenant_name=self.auth_env['OS_TENANT_NAME'],
-            tenant_id='')
+        self.assertEqual(1, self.v2_auth.call_count)
+        self.assertFalse(self.v3_auth.called)
+
+        body = json.loads(self.v2_auth.last_request.body)
+
+        self.assertEqual(self.auth_env['OS_TENANT_NAME'],
+                         body['auth']['tenantName'])
+        self.assertEqual(self.auth_env['OS_USERNAME'],
+                         body['auth']['passwordCredentials']['username'])
+        self.assertEqual(self.auth_env['OS_PASSWORD'],
+                         body['auth']['passwordCredentials']['password'])
 
     @mock.patch('glanceclient.v1.client.Client')
-    @mock.patch('keystoneclient.session.Session')
-    @mock.patch.object(keystoneclient.discover.Discover, 'url_for',
-                       side_effect=[keystone_client_fixtures.V2_URL, None])
-    def test_auth_plugin_invocation_with_v1(self,
-                                            v1_client,
-                                            ks_session,
-                                            url_for):
-        with mock.patch(self.auth_plugin) as mock_auth_plugin:
-            args = 'image-list'
-            glance_shell = openstack_shell.OpenStackImagesShell()
-            glance_shell.main(args.split())
-            self._assert_auth_plugin_args(mock_auth_plugin)
+    def test_auth_plugin_invocation_with_v1(self, v1_client):
+        args = 'image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.v2.client.Client')
-    @mock.patch('keystoneclient.session.Session')
     @mock.patch.object(openstack_shell.OpenStackImagesShell, '_cache_schemas')
-    @mock.patch.object(keystoneclient.discover.Discover, 'url_for',
-                       side_effect=[keystone_client_fixtures.V2_URL, None])
     def test_auth_plugin_invocation_with_v2(self,
                                             v2_client,
-                                            ks_session,
-                                            url_for,
                                             cache_schemas):
-        with mock.patch(self.auth_plugin) as mock_auth_plugin:
-            args = '--os-image-api-version 2 image-list'
-            glance_shell = openstack_shell.OpenStackImagesShell()
-            glance_shell.main(args.split())
-            self._assert_auth_plugin_args(mock_auth_plugin)
+        args = '--os-image-api-version 2 image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.v1.client.Client')
-    @mock.patch('keystoneclient.session.Session')
-    @mock.patch.object(keystoneclient.discover.Discover, 'url_for',
-                       side_effect=[keystone_client_fixtures.V2_URL,
-                                    keystone_client_fixtures.V3_URL])
     def test_auth_plugin_invocation_with_unversioned_auth_url_with_v1(
-            self, v1_client, ks_session, url_for):
-        with mock.patch(self.auth_plugin) as mock_auth_plugin:
-            args = '--os-auth-url %s image-list' % (
-                keystone_client_fixtures.BASE_URL)
-            glance_shell = openstack_shell.OpenStackImagesShell()
-            glance_shell.main(args.split())
-            self._assert_auth_plugin_args(mock_auth_plugin)
+            self, v1_client):
+        args = '--os-auth-url %s image-list' % DEFAULT_UNVERSIONED_AUTH_URL
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.v2.client.Client')
-    @mock.patch('keystoneclient.session.Session')
     @mock.patch.object(openstack_shell.OpenStackImagesShell, '_cache_schemas')
-    @mock.patch.object(keystoneclient.discover.Discover, 'url_for',
-                       side_effect=[keystone_client_fixtures.V2_URL,
-                                    keystone_client_fixtures.V3_URL])
     def test_auth_plugin_invocation_with_unversioned_auth_url_with_v2(
-            self, v2_client, ks_session, cache_schemas, url_for):
-        with mock.patch(self.auth_plugin) as mock_auth_plugin:
-            args = ('--os-auth-url %s --os-image-api-version 2 '
-                    'image-list') % (keystone_client_fixtures.BASE_URL)
-            glance_shell = openstack_shell.OpenStackImagesShell()
-            glance_shell.main(args.split())
-            self._assert_auth_plugin_args(mock_auth_plugin)
+            self, v2_client, cache_schemas):
+        args = ('--os-auth-url %s --os-image-api-version 2 '
+                'image-list') % DEFAULT_UNVERSIONED_AUTH_URL
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self._assert_auth_plugin_args()
 
     @mock.patch('sys.stdin', side_effect=mock.MagicMock)
     @mock.patch('getpass.getpass', return_value='password')
-    @mock.patch('keystoneclient.session.Session.get_token',
-                side_effect=ks_exc.ConnectionRefused)
-    def test_password_prompted_with_v2(self, mock_session, mock_getpass,
-                                       mock_stdin):
+    def test_password_prompted_with_v2(self, mock_getpass, mock_stdin):
+        self.requests.post(self.token_url, exc=requests.ConnectionError)
         glance_shell = openstack_shell.OpenStackImagesShell()
         self.make_env(exclude='OS_PASSWORD')
         self.assertRaises(ks_exc.ConnectionRefused,
@@ -352,60 +363,42 @@ class ShellTest(utils.TestCase):
 class ShellTestWithKeystoneV3Auth(ShellTest):
     # auth environment to use
     auth_env = FAKE_V3_ENV.copy()
-    # expected auth plugin to invoke
-    auth_plugin = 'keystoneclient.auth.identity.v3.Password'
+    token_url = DEFAULT_V3_AUTH_URL + '/auth/tokens'
 
-    def _assert_auth_plugin_args(self, mock_auth_plugin):
-        mock_auth_plugin.assert_called_once_with(
-            keystone_client_fixtures.V3_URL,
-            user_id='',
-            username=self.auth_env['OS_USERNAME'],
-            password=self.auth_env['OS_PASSWORD'],
-            user_domain_id='',
-            user_domain_name=self.auth_env['OS_USER_DOMAIN_NAME'],
-            project_id=self.auth_env['OS_PROJECT_ID'],
-            project_name='',
-            project_domain_id='',
-            project_domain_name='')
+    def _assert_auth_plugin_args(self):
+        self.assertFalse(self.v2_auth.called)
+        self.assertEqual(1, self.v3_auth.call_count)
+
+        body = json.loads(self.v3_auth.last_request.body)
+        user = body['auth']['identity']['password']['user']
+
+        self.assertEqual(self.auth_env['OS_USERNAME'], user['name'])
+        self.assertEqual(self.auth_env['OS_PASSWORD'], user['password'])
+        self.assertEqual(self.auth_env['OS_USER_DOMAIN_NAME'],
+                         user['domain']['name'])
+        self.assertEqual(self.auth_env['OS_PROJECT_ID'],
+                         body['auth']['scope']['project']['id'])
 
     @mock.patch('glanceclient.v1.client.Client')
-    @mock.patch('keystoneclient.session.Session')
-    @mock.patch.object(keystoneclient.discover.Discover, 'url_for',
-                       side_effect=[None, keystone_client_fixtures.V3_URL])
-    def test_auth_plugin_invocation_with_v1(self,
-                                            v1_client,
-                                            ks_session,
-                                            url_for):
-        with mock.patch(self.auth_plugin) as mock_auth_plugin:
-            args = 'image-list'
-            glance_shell = openstack_shell.OpenStackImagesShell()
-            glance_shell.main(args.split())
-            self._assert_auth_plugin_args(mock_auth_plugin)
+    def test_auth_plugin_invocation_with_v1(self, v1_client):
+        args = 'image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self._assert_auth_plugin_args()
 
     @mock.patch('glanceclient.v2.client.Client')
-    @mock.patch('keystoneclient.session.Session')
     @mock.patch.object(openstack_shell.OpenStackImagesShell, '_cache_schemas')
-    @mock.patch.object(keystoneclient.discover.Discover, 'url_for',
-                       side_effect=[None, keystone_client_fixtures.V3_URL])
-    def test_auth_plugin_invocation_with_v2(self,
-                                            v2_client,
-                                            ks_session,
-                                            url_for,
-                                            cache_schemas):
-        with mock.patch(self.auth_plugin) as mock_auth_plugin:
-            args = '--os-image-api-version 2 image-list'
-            glance_shell = openstack_shell.OpenStackImagesShell()
-            glance_shell.main(args.split())
-            self._assert_auth_plugin_args(mock_auth_plugin)
+    def test_auth_plugin_invocation_with_v2(self, v2_client, cache_schemas):
+        args = '--os-image-api-version 2 image-list'
+        glance_shell = openstack_shell.OpenStackImagesShell()
+        glance_shell.main(args.split())
+        self._assert_auth_plugin_args()
 
-    @mock.patch('keystoneclient.session.Session')
     @mock.patch('keystoneclient.discover.Discover',
                 side_effect=ks_exc.ClientException())
     def test_api_discovery_failed_with_unversioned_auth_url(self,
-                                                            ks_session,
                                                             discover):
-        args = '--os-auth-url %s image-list' % (
-            keystone_client_fixtures.BASE_URL)
+        args = '--os-auth-url %s image-list' % DEFAULT_UNVERSIONED_AUTH_URL
         glance_shell = openstack_shell.OpenStackImagesShell()
         self.assertRaises(exc.CommandError, glance_shell.main, args.split())
 
